﻿using System;
using System.Drawing;

namespace Core.Utilities {
	/// <summary>
	/// Libreria per gestione/manipolazione conversione dei colori.
	/// </summary>
	public static class ColorUtility
    {
		/// <summary>
		/// Trasforma l'esadecimale nel tipo Color corrispondente con la trasparenza.
		/// </summary>
		/// <param name="hex">Esadecimale rappresentante il colore (può includere o meno il simbolo #)</param>
		/// <param name="trasparencyPerc">Percentuale di Trasparenza (0-100).</param>
		/// <returns>Ritorna l'oggetto Color corrispondente all'esadecimale.</returns>
		public static Color HexToColor(string hex, int trasparencyPerc = 0) {
            if (hex == null)
                return default(Color);

            // deve includere il #
            if (!hex.StartsWith("#"))
                hex = $"#{hex}";

            if (trasparencyPerc > 0)
                hex = $"#{AlphaToHex(trasparencyPerc)}{hex.Substring(1)}";

            return ColorTranslator.FromHtml(hex);
        }

        /// <summary>
        /// Dato un colore e un valore di opacità da 0 a 255, ritorna il l'oggetto <see cref="Color"/> corrispondente.
        /// </summary>
        /// <param name="sourceColor">Oggetto <see cref="Color"/> sorgente.</param>
        /// <param name="opacity">Valore di opacità da 0 a 255 da applicare al colore passato.</param>
        /// <returns></returns>
        public static Color ApplyOpacityToColor(Color sourceColor, int opacity) {
            return Color.FromArgb(opacity, sourceColor);
        }

		/// <summary>
		/// Data una percentuale di trasparenza (0-100) ritorna la corrispondente conversione in esadecimale
		/// </summary>
		/// <param name="trasparencyPerc">Percentuale di trasparenza (0-100)</param>
		/// <returns>Versione esadecimale della trasparenza passata.</returns>
		public static string AlphaToHex(int transparencyPerc) {
			transparencyPerc = Math.Clamp(transparencyPerc, 0, 100);
			byte alpha = (byte)(transparencyPerc * 255 / 100);
			return alpha.ToString("X2");
		}

		/// <summary>
		/// Data una costante colore e una trasparenza ritorna il colore nella forma esadecimale.
		/// </summary>
		/// <param name="c">Costante <see cref="Color"/></param>
		/// <param name="trasparencyPerc">Percentuale di trasparenza (0-100)</param>
		/// <returns>Versione esadecimale del colore passato</returns>
		public static string ColorToHex(Color c, int trasparencyPerc = 0) {
            if (trasparencyPerc == 0)
                return ColorToHex(c, null);

            return ColorToHex(c, AlphaToHex(trasparencyPerc));
        }

        /// <summary>
        /// Trasforma una costante <see cref="Color" in esadecimale />
        /// </summary>
        /// <param name="c">Costante colore</param>
        /// <param name="trasparency">Trasparenza da 00-ff (0% - 00, 10% - 1A, 25% - 40, 50% - 7F, 75% - BF, 100% - FF)</param>
        /// <returns>Versione esadecimale del colore passato</returns>
        public static string ColorToHex(Color c, string trasparency) {
            if (trasparency == null || trasparency == string.Empty)
                return $"{c.R.ToString("X2")}{c.G.ToString("X2")}{c.B.ToString("X2")}";

            return $"{c.R.ToString("X2")}{c.G.ToString("X2")}{c.B.ToString("X2")}{trasparency}";
        }

		/// <summary>
		/// La funzione AdjustBrightness riceve un oggetto Color e un parametro correctionFactor di tipo float.
		/// Se il correctionFactor è negativo, il colore viene scurito. Se è positivo, viene schiarito. <br></br>
		///  - Un valore di -1 renderebbe il colore completamente nero. <br></br>
		///  - Un valore di 1 renderebbe il colore completamente bianco. <br></br>
		///  - Un valore di 0 restituisce il colore originale senza alcuna modifica. <br></br>
		/// </summary>
		/// <param name="color">L'oggetto color sorgente.</param>
		/// <param name="correctionFactor">Fattore di correzione della lunimosità (negativo = scurisce, positivo = schiarisce, 0 = inalterato).</param>
		/// <returns>L'oggetto color schiarito.</returns>
		public static Color AdjustBrightness(this Color color, float correctionFactor) {
			float red = color.R;
			float green = color.G;
			float blue = color.B;

			if (correctionFactor < 0) {
				correctionFactor = 1 + correctionFactor;
				red *= correctionFactor;
				green *= correctionFactor;
				blue *= correctionFactor;
			}
			else {
				red = (255 - red) * correctionFactor + red;
				green = (255 - green) * correctionFactor + green;
				blue = (255 - blue) * correctionFactor + blue;
			}

			return Color.FromArgb(color.A, (int)red, (int)green, (int)blue);
		}
	}
}
